@@ -28,7 +28,28 @@ def filter_rectangle(
     :param cell_size: size of the tiles to be returned
     :return: a stream of squares (x, y, x+cell_size, y+cell_size)
         for which filter_func(x, y, x+step, y+step) == True
+    """
+    for check, square in discriminate(filter_func, rectangle, cell_size):
+        if check:
+            yield square
 
+
+def discriminate(
+        discrimination_func: Callable[[float, float, float, float], bool],
+        rectangle: Tuple[float, float, float, float],
+        cell_size: float) -> Generator[Tuple[bool, Tuple[float, float, float, float]],
+                                       None, None]:
+    """Given a rectangle, return a stream of squares alongside the value of
+    the discrimination function on that square:
+    * squares where the function is True will be of size cell_size;
+    * square where the function is False will be of size cell_size * 2^i,
+      with i >= 0.
+
+    :param discrimination_func: a filter function
+    :param rectangle: the rectangle (x0, y0, x1, y1)
+    :param cell_size: size of the squares to be returned for these squares
+        where filter_func evaluates to True
+    :return: a stream of results and squares
     """
     x0, y0, x1, y1 = rectangle
 
@@ -44,7 +65,7 @@ def filter_rectangle(
             f"got cell size={cell_size}")
 
     square_size, tree_depth = _get_shape_parameters(x0, y0, x1, y1, cell_size)
-    yield from _filter_square(filter_func, x0, y0, square_size, tree_depth)
+    yield from _filter_square(discrimination_func, x0, y0, square_size, tree_depth)
 
 
 def _get_shape_parameters(x0, y0, x1, y1, cell_size) -> Tuple[float, int]:
@@ -66,7 +87,7 @@ def _filter_square(
         if tree_depth == 0:
             # The filter is true and this is the smallest square,
             # return it.
-            yield x0, y0, x0 + side_length, y0 + side_length
+            yield True, (x0, y0, x0 + side_length, y0 + side_length)
         else:
             # This is not the smallest square yet, cut it in 4 smaller
             # squares and test those.
@@ -77,4 +98,6 @@ def _filter_square(
                     small_y = y0 + step_j * half_length
                     yield from _filter_square(
                         filter_func, small_x, small_y,
-                        half_length, tree_depth-1)
+                        half_length, tree_depth - 1)
+    else:
+        yield False, (x0, y0, x0 + side_length, y0 + side_length)
